import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
const _excluded = ["children", "className", "component", "columnSpan", "defaultHeight"];
import * as React from 'react';
import PropTypes from 'prop-types';
import clsx from 'clsx';
import { createUnarySpacing, getValue, handleBreakpoints, unstable_resolveBreakpointValues as resolveBreakpointValues } from '@mui/system';
import { unstable_useForkRef as useForkRef } from '@mui/utils';
import { unstable_composeClasses as composeClasses } from '@mui/core';
import { styled, useThemeProps, useTheme } from '@mui/material/styles';
import { getMasonryItemUtilityClass } from './masonryItemClasses';
import MasonryContext from '../Masonry/MasonryContext';
import { jsx as _jsx } from "react/jsx-runtime";

const useUtilityClasses = ownerState => {
  const {
    classes
  } = ownerState;
  const slots = {
    root: ['root']
  };
  return composeClasses(slots, getMasonryItemUtilityClass, classes);
};

export const style = ({
  ownerState,
  theme
}) => {
  let styles = {
    width: '100%',
    '& > *': _extends({
      // all contents should have a width of 100%
      width: '100%',
      boxSizing: 'inherit'
    }, ownerState.hasDefaultHeight && {
      height: '100%'
    }),
    visibility: ownerState.height ? 'visible' : 'hidden',
    gridColumnEnd: `span ${ownerState.columnSpan}`,
    boxSizing: 'inherit'
  };

  if (Array.isArray(ownerState.spacing) || typeof ownerState.spacing === 'object') {
    const base = {};
    Object.keys(theme.breakpoints.values).forEach(breakpoint => {
      if (ownerState.spacing[breakpoint] != null) {
        base[breakpoint] = true;
      }
    });
    const spacingValues = resolveBreakpointValues({
      values: ownerState.spacing,
      base
    });
    const transformer = createUnarySpacing(theme);

    const styleFromPropValue = propValue => {
      const gap = ownerState.height ? Number(getValue(transformer, propValue).replace('px', '')) : 0; // For lazy-loaded images to load properly, masonry item should take up space greater than 1px.
      // Taking into account a row gap of 2px, rowSpan should at least be 2.

      const rowSpan = ownerState.height ? Math.ceil((ownerState.height + gap) / 2) : 2;
      return {
        gridRowEnd: `span ${rowSpan}`,
        paddingBottom: gap === 0 ? 0 : gap - 2
      };
    };

    styles = _extends({}, styles, handleBreakpoints({
      theme
    }, spacingValues, styleFromPropValue));
  }

  return styles;
};
const MasonryItemRoot = styled('div', {
  name: 'MuiMasonryItem',
  slot: 'Root',
  overridesResolver: (props, styles) => {
    return [styles.root];
  }
})(style);
const MasonryItem = /*#__PURE__*/React.forwardRef(function MasonryItem(inProps, ref) {
  const props = useThemeProps({
    props: inProps,
    name: 'MuiMasonryItem'
  });
  const masonryItemRef = React.useRef(null);
  const {
    spacing = 1
  } = React.useContext(MasonryContext);

  const {
    children,
    className,
    component = 'div',
    columnSpan = 1,
    defaultHeight
  } = props,
        other = _objectWithoutPropertiesLoose(props, _excluded);

  const hasDefaultHeight = defaultHeight !== undefined;
  const [height, setHeight] = React.useState(defaultHeight);

  const ownerState = _extends({}, props, {
    spacing,
    hasDefaultHeight,
    columnSpan,
    height: height < 0 ? 0 : height // MasonryItems to which negative or zero height is passed will be hidden

  });

  const classes = useUtilityClasses(ownerState);
  React.useEffect(() => {
    // Do not create a resize observer in case of provided height masonry
    if (hasDefaultHeight) {
      return undefined;
    }

    if (typeof ResizeObserver === 'undefined') {
      return undefined;
    }

    const resizeObserver = new ResizeObserver(([item]) => {
      setHeight(item.contentRect.height);
    });
    resizeObserver.observe(masonryItemRef.current.firstChild);
    return () => {
      resizeObserver.disconnect();
    };
  }, [hasDefaultHeight]);
  const handleRef = useForkRef(ref, masonryItemRef);
  const theme = useTheme();
  const styleProp = {};

  if (!Array.isArray(spacing) && typeof spacing !== 'object') {
    const gap = height ? Number(theme.spacing(spacing).replace('px', '')) : 0;
    const rowSpan = height ? Math.ceil((height + gap) / 2) : 2;
    styleProp.gridRowEnd = `span ${rowSpan}`;
    styleProp.paddingBottom = gap === 0 ? 0 : gap - 2;
  }

  return /*#__PURE__*/_jsx(MasonryItemRoot, _extends({
    as: component,
    className: clsx(classes.root, className),
    ref: handleRef,
    ownerState: ownerState,
    style: styleProp
  }, other, {
    children: React.Children.only(children)
  }));
});
process.env.NODE_ENV !== "production" ? MasonryItem.propTypes
/* remove-proptypes */
= {
  // ----------------------------- Warning --------------------------------
  // | These PropTypes are generated from the TypeScript type definitions |
  // |     To update them edit the d.ts file and run "yarn proptypes"     |
  // ----------------------------------------------------------------------

  /**
   * The content of the component, normally an `<img />` or a `<div />`. It should be only one element.
   */
  children: PropTypes.element.isRequired,

  /**
   * Override or extend the styles applied to the component.
   */
  classes: PropTypes.object,

  /**
   * @ignore
   */
  className: PropTypes.string,

  /**
   * The number of columns taken up by the component
   * @default 1
   */
  columnSpan: PropTypes.number,

  /**
   * The component used for the root node.
   * Either a string to use a HTML element or a component.
   */
  component: PropTypes.elementType,

  /**
   * The initial height of the component in px. This is provided for server-side rendering.
   */
  defaultHeight: PropTypes.number,

  /**
   * Allows defining system overrides as well as additional CSS styles.
   */
  sx: PropTypes.object
} : void 0;
export default MasonryItem;